# -*- coding: utf-8 -*-
#
#  Cloud-Control Node
#  Copyright (C) 2011 SmartJog [http://www.smartjog.com]
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#  Authors:
#    * 2011-06: Thibault VINCENT <thibault.vincent@smartjog.com>
#

from __future__ import absolute_import
from subprocess import Popen, PIPE, STDOUT
from ..kernel import BaseModule, ModuleError, Handler, pure


class Module(BaseModule):

    _handler_methods = {
        'execute_command' : '_h_execute_command',
    }

    def init(self, manager):

        self._manager = manager
        self._logger = manager.get_logger()
        self._config = manager.get_configuration()

        # create and populate our own handler
        self._handler = Handler(self._logger, 'execute')
        for name, method in self._handler_methods.iteritems():
            self._handler.procedure_register(name, getattr(self, method))
        # register it in the core handler
        self._manager.get_handler().register(self._handler)

    def destroy(self):

        # FIXME do something about running commands (wait? kill?)
        
        # unregister ourselves from the core handler
        self._manager.get_handler().unregister(self._handler)

    @pure
    def _h_execute_command(self, command):

        # do nothing when execution is disabled in configuration
        if self._config.get('command_execution') is not True:
            raise ModuleError('command execution is disabled by configuration')

        # synchronous execution
        self._logger.info('executing command: %s', command)
        output = None
        try:
            output = Popen(command, shell=True, bufsize=-1, stdin=PIPE,
                                stdout=PIPE, stderr=STDOUT).communicate()[0]
        # command not found or related error
        except OSError as err:
            raise ModuleError('could not execute command: %s' % err)
        # return output string, mix of stdout and stderr
        else:
            return output
