# -*- coding: utf-8 -*-
#
#  Cloud-Control Node
#  Copyright (C) 2011 SmartJog [http://www.smartjog.com]
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#  Authors:
#    * 2011-06: Thibault VINCENT <thibault.vincent@smartjog.com>
#

from __future__ import absolute_import
from subprocess import Popen, PIPE, STDOUT
from ..kernel import BaseModule, ModuleError, Handler, pure


class Module(BaseModule):

    _handler_methods = {
        'node_shutdown' : '_h_node_shutdown',
    }

    def init(self, manager):

        self._manager = manager
        self._logger = manager.get_logger()
        self._config = manager.get_configuration()

        # create and populate our own handler
        self._handler = Handler(self._logger, 'shutdown')
        for name, method in self._handler_methods.iteritems():
            self._handler.procedure_register(name, getattr(self, method))
        # register it in the core handler
        self._manager.get_handler().register(self._handler)

    def destroy(self):
        
        # unregister ourselves from the core handler
        self._manager.get_handler().unregister(self._handler)

    @pure
    def _h_node_shutdown(self, reboot=True, gracefull=True):

        self._logger.info('server requested shutdown of local host with'
                    ' options reboot=`%s` gracefull=`%s`', reboot, gracefull)
        
        # build option list for the GNU shutdown command
        options = ''
        if reboot is False:
            options += ' -h -P'
            options += '' if gracefull else ' -n'
        else:
            options += ' -r'
            options += ' -f' if gracefull else ' -n'
        options += ' 0'
        
        # shutdown machine
        try:
            Popen('/sbin/shutdown %s' % options, shell=True).communicate()
        # command not found or related error
        except OSError as err:
            raise ModuleError('could not execute shutdown: %s' % err)
