# -*- coding: utf-8 -*-
#
#  Cloud-Control Node
#  Copyright (C) 2011 SmartJog [http://www.smartjog.com]
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#  Authors:
#    * 2011-06: Thibault VINCENT <thibault.vincent@smartjog.com>
#

from __future__ import absolute_import
from threading import Lock


class RWLock(object):
    # pylint: disable=R0903
    
    def __init__(self):
        
        self._mutex = Lock()
        self._writemutex = Lock()
        self._readers = 0
        self._writers = 0
        self.read = self._RLock(self)
        self.write = self._WLock(self)
    
    
    class _Lock(object):
        
        def __init__(self, rwlock):
            
            self._parent = rwlock
    
    
    class _WLock(_Lock):
        
        def __enter__(self):
            
            self.acquire()
        
        def __exit__(self, exc_type, exc_value, traceback):
            
            self.release()
        
        def acquire(self):
            # pylint: disable=W0212
            
            with self._parent._mutex:
                self._parent._writers += 1
            self._parent._writemutex.acquire()
        
        def release(self):
            # pylint: disable=W0212
            
            with self._parent._mutex:
                self._parent._writers -= 1
            self._parent._writemutex.release()
    
    
    class _RLock(_Lock):
        
        def __enter__(self):
            
            self.acquire()
        
        def __exit__(self, exc_type, exc_value, traceback):
            
            self.release()
        
        def acquire(self):
            # pylint: disable=W0212
            
            self._parent._mutex.acquire()
            if self._parent._writers > 0 or self._parent._readers == 0:
                self._parent._mutex.release()
                self._parent._writemutex.acquire()
                self._parent._mutex.acquire()
            self._parent._readers += 1
            self._parent._mutex.release()
        
        def release(self):
            # pylint: disable=W0212
            
            self._parent._mutex.acquire()
            self._parent._readers -= 1
            if self._parent._readers == 0:
                self._parent._writemutex.release()
            self._parent._mutex.release()
